//
//  Copyright (c) 2003 Launchbird Design Systems, Inc.
//  All rights reserved.
//  
//  Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
//    Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
//    Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
//  
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
//  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
//  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
//  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
//  OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
//  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//  
//  
//  Overview:
//  
//    Performs finite impulse response (FIR) filtering.
//    The filter's sum of products is pipelined with a register after
//    every multiplier and adder.  The adder network is a balanced binary
//    tree to minimize latency.  The sum of products has no numeric loss because the
//    multipliers keep all resulting bits and each adder extends the precision by 1.
//  
//  Interface:
//  
//    Synchronization:
//      clock_c  : Clock input.
//      reset_i  : Filter delay bank synchronous reset.  Does not reset sum of products pipeline registers.
//  
//    Inputs:
//      data_i  : Input data.
//      k0_i    : Coefficient 0 multiplied by in_i(0).
//      k1_i    : Coefficient 1 multiplied by in_i(k-1).
//      k2_i    : Coefficient 2 multiplied by in_i(k-2).
//      ...
//      k<order>_i : Coefficient <order> multiplied by in_i(k-<order>).
//  
//    Outputs:
//      data_o : Output data.
//  
//  Built In Parameters:
//  
//    Filter Order             = 3
//    Input Precision          = 8
//    Coefficient Precision    = 8
//    Sum of Products Latency  = 3
//  
//  
//  
//  
//  Generated by Confluence 0.6.3  --  Launchbird Design Systems, Inc.  --  www.launchbird.com
//  
//  Build Date : Fri Aug 22 09:45:46 CDT 2003
//  
//  Interface
//  
//    Build Name    : cf_fir_3_8_8
//    Clock Domains : clock_c  
//    Vector Input  : reset_i(1)
//    Vector Input  : data_i(8)
//    Vector Input  : k0_i(8)
//    Vector Input  : k1_i(8)
//    Vector Input  : k2_i(8)
//    Vector Input  : k3_i(8)
//    Vector Output : data_o(18)
//  
//  
//  

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include "cf_fir_3_8_8.h"

static unsigned char *data_o;
static unsigned char n95[1];
static unsigned char n93[1];
static unsigned char n83[1];
static unsigned char n81[1];
static unsigned char n71[1];
static unsigned char n69[1];
static unsigned char *data_i;
static unsigned char *k3_i;
static unsigned char *k2_i;
static unsigned char *k1_i;
static unsigned char *k0_i;
static unsigned char *reset_i;
static unsigned char n2[1];
static unsigned char n1[1];
static unsigned char n96[3];
static unsigned char n94[3];
static unsigned char n84[3];
static unsigned char n82[3];
static unsigned char n72[3];
static unsigned char n70[3];
static unsigned char n61[2];
static unsigned char n53[2];
static unsigned char n45[2];
static unsigned char n37[2];
static unsigned char n36[1];
static unsigned char n36r[1];
static unsigned char n36e[1];
static unsigned char n36d[1];
static unsigned char n29[1];
static unsigned char n29r[1];
static unsigned char n29e[1];
static unsigned char n29d[1];
static unsigned char n22[1];
static unsigned char n22r[1];
static unsigned char n22e[1];
static unsigned char n22d[1];
static unsigned char n15[1];
static unsigned char n15r[1];
static unsigned char n15e[1];
static unsigned char n15d[1];
static unsigned char n97[3];
static unsigned char n85[3];
static unsigned char n73[3];
static unsigned char n68[2];
static unsigned char n68r[1];
static unsigned char n68e[1];
static unsigned char n68d[2];
static unsigned char n60[2];
static unsigned char n60r[1];
static unsigned char n60e[1];
static unsigned char n60d[2];
static unsigned char n52[2];
static unsigned char n52r[1];
static unsigned char n52e[1];
static unsigned char n52d[2];
static unsigned char n44[2];
static unsigned char n44r[1];
static unsigned char n44e[1];
static unsigned char n44d[2];
static unsigned char n104[3];
static unsigned char n104r[1];
static unsigned char n104e[1];
static unsigned char n104d[3];
static unsigned char n92[3];
static unsigned char n92r[1];
static unsigned char n92e[1];
static unsigned char n92d[3];
static unsigned char n80[3];
static unsigned char n80r[1];
static unsigned char n80e[1];
static unsigned char n80d[3];

void cf_fir_3_8_8_ports(unsigned char* port_reset_i, unsigned char* port_data_i, unsigned char* port_k0_i, unsigned char* port_k1_i, unsigned char* port_k2_i, unsigned char* port_k3_i, unsigned char* port_data_o) {
  reset_i = port_reset_i;
  data_i = port_data_i;
  k0_i = port_k0_i;
  k1_i = port_k1_i;
  k2_i = port_k2_i;
  k3_i = port_k3_i;
  data_o = port_data_o;
}

void cf_fir_3_8_8_calc() {
  data_o[0] = n104[0];
  data_o[1] = n104[1];
  data_o[2] = n104[2];
  n95[0] = (1 & (n92[2]));
  n93[0] = (1 & (n80[2]));
  n83[0] = (1 & (n68[1] >> 7));
  n81[0] = (1 & (n60[1] >> 7));
  n71[0] = (1 & (n52[1] >> 7));
  n69[0] = (1 & (n44[1] >> 7));
  n2[0] = 0;
  n1[0] = 1;
  n96[0] = n92[0];
  n96[1] = n92[1];
  n96[2] = 255 & (n95[0] << 1 | n92[2]);
  n94[0] = n80[0];
  n94[1] = n80[1];
  n94[2] = 255 & (n93[0] << 1 | n80[2]);
  n84[0] = n68[0];
  n84[1] = n68[1];
  n84[2] = n83[0];
  n82[0] = n60[0];
  n82[1] = n60[1];
  n82[2] = n81[0];
  n72[0] = n52[0];
  n72[1] = n52[1];
  n72[2] = n71[0];
  n70[0] = n44[0];
  n70[1] = n44[1];
  n70[2] = n69[0];
  {
    unsigned int tmp;
    unsigned char a[2];
    unsigned char b[2];
    a[1] = 0;
    a[0] = k3_i[0];
    b[1] = 0;
    b[0] = n36[0];
    (tmp = ((unsigned int) a[1] * (unsigned int) b[0]) + ((unsigned int) a[0] * (unsigned int) b[1]) + (tmp = ((unsigned int) a[0] * (unsigned int) b[0]), n61[0] = 255 & (unsigned char) tmp, tmp >> 8), n61[1] = 255 & (unsigned char) tmp);
  }
  {
    unsigned int tmp;
    unsigned char a[2];
    unsigned char b[2];
    a[1] = 0;
    a[0] = k2_i[0];
    b[1] = 0;
    b[0] = n29[0];
    (tmp = ((unsigned int) a[1] * (unsigned int) b[0]) + ((unsigned int) a[0] * (unsigned int) b[1]) + (tmp = ((unsigned int) a[0] * (unsigned int) b[0]), n53[0] = 255 & (unsigned char) tmp, tmp >> 8), n53[1] = 255 & (unsigned char) tmp);
  }
  {
    unsigned int tmp;
    unsigned char a[2];
    unsigned char b[2];
    a[1] = 0;
    a[0] = k1_i[0];
    b[1] = 0;
    b[0] = n22[0];
    (tmp = ((unsigned int) a[1] * (unsigned int) b[0]) + ((unsigned int) a[0] * (unsigned int) b[1]) + (tmp = ((unsigned int) a[0] * (unsigned int) b[0]), n45[0] = 255 & (unsigned char) tmp, tmp >> 8), n45[1] = 255 & (unsigned char) tmp);
  }
  {
    unsigned int tmp;
    unsigned char a[2];
    unsigned char b[2];
    a[1] = 0;
    a[0] = k0_i[0];
    b[1] = 0;
    b[0] = n15[0];
    (tmp = ((unsigned int) a[1] * (unsigned int) b[0]) + ((unsigned int) a[0] * (unsigned int) b[1]) + (tmp = ((unsigned int) a[0] * (unsigned int) b[0]), n37[0] = 255 & (unsigned char) tmp, tmp >> 8), n37[1] = 255 & (unsigned char) tmp);
  }
  n36r[0] = reset_i[0];
  n36e[0] = n1[0];
  n36d[0] = n29[0];
  n29r[0] = reset_i[0];
  n29e[0] = n1[0];
  n29d[0] = n22[0];
  n22r[0] = reset_i[0];
  n22e[0] = n1[0];
  n22d[0] = n15[0];
  n15r[0] = reset_i[0];
  n15e[0] = n1[0];
  n15d[0] = data_i[0];
  { unsigned int tmp; (tmp = (unsigned int) n94[2] + (unsigned int) n96[2] + (tmp = (unsigned int) n94[1] + (unsigned int) n96[1] + (tmp = (unsigned int) n94[0] + (unsigned int) n96[0], n97[0] = 255 & (unsigned char) tmp, 1 & (tmp >> 8)), n97[1] = 255 & (unsigned char) tmp, 1 & (tmp >> 8)), n97[2] = 3 & (unsigned char) tmp); }
  { unsigned int tmp; (tmp = (unsigned int) n82[2] + (unsigned int) n84[2] + (tmp = (unsigned int) n82[1] + (unsigned int) n84[1] + (tmp = (unsigned int) n82[0] + (unsigned int) n84[0], n85[0] = 255 & (unsigned char) tmp, 1 & (tmp >> 8)), n85[1] = 255 & (unsigned char) tmp, 1 & (tmp >> 8)), n85[2] = 1 & (unsigned char) tmp); }
  { unsigned int tmp; (tmp = (unsigned int) n70[2] + (unsigned int) n72[2] + (tmp = (unsigned int) n70[1] + (unsigned int) n72[1] + (tmp = (unsigned int) n70[0] + (unsigned int) n72[0], n73[0] = 255 & (unsigned char) tmp, 1 & (tmp >> 8)), n73[1] = 255 & (unsigned char) tmp, 1 & (tmp >> 8)), n73[2] = 1 & (unsigned char) tmp); }
  n68r[0] = n2[0];
  n68e[0] = n1[0];
  n68d[0] = n61[0];
  n68d[1] = n61[1];
  n60r[0] = n2[0];
  n60e[0] = n1[0];
  n60d[0] = n53[0];
  n60d[1] = n53[1];
  n52r[0] = n2[0];
  n52e[0] = n1[0];
  n52d[0] = n45[0];
  n52d[1] = n45[1];
  n44r[0] = n2[0];
  n44e[0] = n1[0];
  n44d[0] = n37[0];
  n44d[1] = n37[1];
  n104r[0] = n2[0];
  n104e[0] = n1[0];
  n104d[0] = n97[0];
  n104d[1] = n97[1];
  n104d[2] = n97[2];
  n92r[0] = n2[0];
  n92e[0] = n1[0];
  n92d[0] = n85[0];
  n92d[1] = n85[1];
  n92d[2] = n85[2];
  n80r[0] = n2[0];
  n80e[0] = n1[0];
  n80d[0] = n73[0];
  n80d[1] = n73[1];
  n80d[2] = n73[2];
}

void cf_fir_3_8_8_init() {
  n95[0] = 0;
  n93[0] = 0;
  n83[0] = 0;
  n81[0] = 0;
  n71[0] = 0;
  n69[0] = 0;
  n2[0] = 0;
  n1[0] = 0;
  { int index; for (index = 3; index > 0;) n96[--index] = 0; }
  { int index; for (index = 3; index > 0;) n94[--index] = 0; }
  { int index; for (index = 3; index > 0;) n84[--index] = 0; }
  { int index; for (index = 3; index > 0;) n82[--index] = 0; }
  { int index; for (index = 3; index > 0;) n72[--index] = 0; }
  { int index; for (index = 3; index > 0;) n70[--index] = 0; }
  { int index; for (index = 2; index > 0;) n61[--index] = 0; }
  { int index; for (index = 2; index > 0;) n53[--index] = 0; }
  { int index; for (index = 2; index > 0;) n45[--index] = 0; }
  { int index; for (index = 2; index > 0;) n37[--index] = 0; }
  n36[0] = 0;
  n36r[0] = 0;
  n36e[0] = 0;
  n36d[0] = 0;
  n29[0] = 0;
  n29r[0] = 0;
  n29e[0] = 0;
  n29d[0] = 0;
  n22[0] = 0;
  n22r[0] = 0;
  n22e[0] = 0;
  n22d[0] = 0;
  n15[0] = 0;
  n15r[0] = 0;
  n15e[0] = 0;
  n15d[0] = 0;
  { int index; for (index = 3; index > 0;) n97[--index] = 0; }
  { int index; for (index = 3; index > 0;) n85[--index] = 0; }
  { int index; for (index = 3; index > 0;) n73[--index] = 0; }
  { int index; for (index = 2; index > 0;) n68[--index] = 0; }
  n68r[0] = 0;
  n68e[0] = 0;
  { int index; for (index = 2; index > 0;) n68d[--index] = 0; }
  { int index; for (index = 2; index > 0;) n60[--index] = 0; }
  n60r[0] = 0;
  n60e[0] = 0;
  { int index; for (index = 2; index > 0;) n60d[--index] = 0; }
  { int index; for (index = 2; index > 0;) n52[--index] = 0; }
  n52r[0] = 0;
  n52e[0] = 0;
  { int index; for (index = 2; index > 0;) n52d[--index] = 0; }
  { int index; for (index = 2; index > 0;) n44[--index] = 0; }
  n44r[0] = 0;
  n44e[0] = 0;
  { int index; for (index = 2; index > 0;) n44d[--index] = 0; }
  { int index; for (index = 3; index > 0;) n104[--index] = 0; }
  n104r[0] = 0;
  n104e[0] = 0;
  { int index; for (index = 3; index > 0;) n104d[--index] = 0; }
  { int index; for (index = 3; index > 0;) n92[--index] = 0; }
  n92r[0] = 0;
  n92e[0] = 0;
  { int index; for (index = 3; index > 0;) n92d[--index] = 0; }
  { int index; for (index = 3; index > 0;) n80[--index] = 0; }
  n80r[0] = 0;
  n80e[0] = 0;
  { int index; for (index = 3; index > 0;) n80d[--index] = 0; }
  cf_fir_3_8_8_calc();
}

void cf_fir_3_8_8_cycle_clock()
{
  if (n15r[0]) {
    n15[0] = 0;
  }
  else if (n15e[0]) {
    n15[0] = n15d[0];
  }
  if (n22r[0]) {
    n22[0] = 0;
  }
  else if (n22e[0]) {
    n22[0] = n22d[0];
  }
  if (n29r[0]) {
    n29[0] = 0;
  }
  else if (n29e[0]) {
    n29[0] = n29d[0];
  }
  if (n36r[0]) {
    n36[0] = 0;
  }
  else if (n36e[0]) {
    n36[0] = n36d[0];
  }
  if (n44r[0]) {
    n44[0] = 0;
    n44[1] = 0;
  }
  else if (n44e[0]) {
    n44[0] = n44d[0];
    n44[1] = n44d[1];
  }
  if (n52r[0]) {
    n52[0] = 0;
    n52[1] = 0;
  }
  else if (n52e[0]) {
    n52[0] = n52d[0];
    n52[1] = n52d[1];
  }
  if (n60r[0]) {
    n60[0] = 0;
    n60[1] = 0;
  }
  else if (n60e[0]) {
    n60[0] = n60d[0];
    n60[1] = n60d[1];
  }
  if (n68r[0]) {
    n68[0] = 0;
    n68[1] = 0;
  }
  else if (n68e[0]) {
    n68[0] = n68d[0];
    n68[1] = n68d[1];
  }
  if (n80r[0]) {
    n80[0] = 0;
    n80[1] = 0;
    n80[2] = 0;
  }
  else if (n80e[0]) {
    n80[0] = n80d[0];
    n80[1] = n80d[1];
    n80[2] = n80d[2];
  }
  if (n92r[0]) {
    n92[0] = 0;
    n92[1] = 0;
    n92[2] = 0;
  }
  else if (n92e[0]) {
    n92[0] = n92d[0];
    n92[1] = n92d[1];
    n92[2] = n92d[2];
  }
  if (n104r[0]) {
    n104[0] = 0;
    n104[1] = 0;
    n104[2] = 0;
  }
  else if (n104e[0]) {
    n104[0] = n104d[0];
    n104[1] = n104d[1];
    n104[2] = n104d[2];
  }
}

static FILE* sim_file;
static int sim_count;
static unsigned char sim_reset_i[1];
static unsigned char sim_data_i[1];
static unsigned char sim_k0_i[1];
static unsigned char sim_k1_i[1];
static unsigned char sim_k2_i[1];
static unsigned char sim_k3_i[1];
static unsigned char sim_data_o[3];

void cf_fir_3_8_8_sim_init(const char* file) {
  time_t date = time(NULL);
  sim_count = 1;
  cf_fir_3_8_8_init();
  sim_reset_i[0] = reset_i[0];
  sim_data_i[0] = data_i[0];
  sim_k0_i[0] = k0_i[0];
  sim_k1_i[0] = k1_i[0];
  sim_k2_i[0] = k2_i[0];
  sim_k3_i[0] = k3_i[0];
  sim_data_o[0] = data_o[0];
  sim_data_o[1] = data_o[1];
  sim_data_o[2] = data_o[2];
  sim_file = fopen(file, "w");
  fprintf(sim_file,
    "$date\n  %s$end\n"
    "$version\n  Confluence 0.6.3 -- Launchbird Design Systems, Inc.\n$end\n"
    "$timescale\n  1 ns\n$end\n"
    "$scope module cf_fir_3_8_8 $end\n"
    "$var wire 1 ! reset_i $end\n"
    "$var wire 8 \" data_i $end\n"
    "$var wire 8 # k0_i $end\n"
    "$var wire 8 $ k1_i $end\n"
    "$var wire 8 %% k2_i $end\n"
    "$var wire 8 & k3_i $end\n"
    "$var wire 18 ' data_o $end\n"
    "$upscope $end\n"
    "$enddefinitions $end\n"
    "#0\n"
    "$dumpvars\n", asctime(localtime(&date)));
  fprintf(sim_file, reset_i[0] ? "1!\n" : "0!\n");
  {
    int bit;
    int byte;
    int found = 0;
    for (byte = 0; byte >= 0; byte--) {
      for (bit = 128; bit >= 1; bit /= 2) {
        if (found)
          fprintf(sim_file, (bit & data_i[byte]) ? "1" : "0");
        else if (bit & data_i[byte]) {
          found = 1;
          fprintf(sim_file, "b1");
        }
      }
    }
    if (!found)
      fprintf(sim_file, "b0");
    fprintf(sim_file, " \"\n");
  }
  {
    int bit;
    int byte;
    int found = 0;
    for (byte = 0; byte >= 0; byte--) {
      for (bit = 128; bit >= 1; bit /= 2) {
        if (found)
          fprintf(sim_file, (bit & k0_i[byte]) ? "1" : "0");
        else if (bit & k0_i[byte]) {
          found = 1;
          fprintf(sim_file, "b1");
        }
      }
    }
    if (!found)
      fprintf(sim_file, "b0");
    fprintf(sim_file, " #\n");
  }
  {
    int bit;
    int byte;
    int found = 0;
    for (byte = 0; byte >= 0; byte--) {
      for (bit = 128; bit >= 1; bit /= 2) {
        if (found)
          fprintf(sim_file, (bit & k1_i[byte]) ? "1" : "0");
        else if (bit & k1_i[byte]) {
          found = 1;
          fprintf(sim_file, "b1");
        }
      }
    }
    if (!found)
      fprintf(sim_file, "b0");
    fprintf(sim_file, " $\n");
  }
  {
    int bit;
    int byte;
    int found = 0;
    for (byte = 0; byte >= 0; byte--) {
      for (bit = 128; bit >= 1; bit /= 2) {
        if (found)
          fprintf(sim_file, (bit & k2_i[byte]) ? "1" : "0");
        else if (bit & k2_i[byte]) {
          found = 1;
          fprintf(sim_file, "b1");
        }
      }
    }
    if (!found)
      fprintf(sim_file, "b0");
    fprintf(sim_file, " %%\n");
  }
  {
    int bit;
    int byte;
    int found = 0;
    for (byte = 0; byte >= 0; byte--) {
      for (bit = 128; bit >= 1; bit /= 2) {
        if (found)
          fprintf(sim_file, (bit & k3_i[byte]) ? "1" : "0");
        else if (bit & k3_i[byte]) {
          found = 1;
          fprintf(sim_file, "b1");
        }
      }
    }
    if (!found)
      fprintf(sim_file, "b0");
    fprintf(sim_file, " &\n");
  }
  {
    int bit;
    int byte;
    int found = 0;
    for (byte = 2; byte >= 0; byte--) {
      for (bit = 128; bit >= 1; bit /= 2) {
        if (found)
          fprintf(sim_file, (bit & data_o[byte]) ? "1" : "0");
        else if (bit & data_o[byte]) {
          found = 1;
          fprintf(sim_file, "b1");
        }
      }
    }
    if (!found)
      fprintf(sim_file, "b0");
    fprintf(sim_file, " '\n");
  }
  fprintf(sim_file, "$end\n");
}

void cf_fir_3_8_8_sim_end() {
  fprintf(sim_file, "#%d\n", sim_count);
  fclose(sim_file);
}

void cf_fir_3_8_8_sim_sample() {
  int changed = 0;
  if (sim_reset_i[0] != reset_i[0]) {
    if (!changed) { changed = 1; fprintf(sim_file, "#%d\n", sim_count); }
    fprintf(sim_file, reset_i[0] ? "1!\n" : "0!\n");
    sim_reset_i[0] = reset_i[0];
  }
  if (sim_data_i[0] != data_i[0]) {
    if (!changed) { changed = 1; fprintf(sim_file, "#%d\n", sim_count); }
    {
      int bit;
      int byte;
      int found = 0;
      for (byte = 0; byte >= 0; byte--) {
        for (bit = 128; bit >= 1; bit /= 2) {
          if (found)
            fprintf(sim_file, (bit & data_i[byte]) ? "1" : "0");
          else if (bit & data_i[byte]) {
            found = 1;
            fprintf(sim_file, "b1");
          }
        }
      }
      if (!found)
        fprintf(sim_file, "b0");
      fprintf(sim_file, " \"\n");
    }
    sim_data_i[0] = data_i[0];
  }
  if (sim_k0_i[0] != k0_i[0]) {
    if (!changed) { changed = 1; fprintf(sim_file, "#%d\n", sim_count); }
    {
      int bit;
      int byte;
      int found = 0;
      for (byte = 0; byte >= 0; byte--) {
        for (bit = 128; bit >= 1; bit /= 2) {
          if (found)
            fprintf(sim_file, (bit & k0_i[byte]) ? "1" : "0");
          else if (bit & k0_i[byte]) {
            found = 1;
            fprintf(sim_file, "b1");
          }
        }
      }
      if (!found)
        fprintf(sim_file, "b0");
      fprintf(sim_file, " #\n");
    }
    sim_k0_i[0] = k0_i[0];
  }
  if (sim_k1_i[0] != k1_i[0]) {
    if (!changed) { changed = 1; fprintf(sim_file, "#%d\n", sim_count); }
    {
      int bit;
      int byte;
      int found = 0;
      for (byte = 0; byte >= 0; byte--) {
        for (bit = 128; bit >= 1; bit /= 2) {
          if (found)
            fprintf(sim_file, (bit & k1_i[byte]) ? "1" : "0");
          else if (bit & k1_i[byte]) {
            found = 1;
            fprintf(sim_file, "b1");
          }
        }
      }
      if (!found)
        fprintf(sim_file, "b0");
      fprintf(sim_file, " $\n");
    }
    sim_k1_i[0] = k1_i[0];
  }
  if (sim_k2_i[0] != k2_i[0]) {
    if (!changed) { changed = 1; fprintf(sim_file, "#%d\n", sim_count); }
    {
      int bit;
      int byte;
      int found = 0;
      for (byte = 0; byte >= 0; byte--) {
        for (bit = 128; bit >= 1; bit /= 2) {
          if (found)
            fprintf(sim_file, (bit & k2_i[byte]) ? "1" : "0");
          else if (bit & k2_i[byte]) {
            found = 1;
            fprintf(sim_file, "b1");
          }
        }
      }
      if (!found)
        fprintf(sim_file, "b0");
      fprintf(sim_file, " %%\n");
    }
    sim_k2_i[0] = k2_i[0];
  }
  if (sim_k3_i[0] != k3_i[0]) {
    if (!changed) { changed = 1; fprintf(sim_file, "#%d\n", sim_count); }
    {
      int bit;
      int byte;
      int found = 0;
      for (byte = 0; byte >= 0; byte--) {
        for (bit = 128; bit >= 1; bit /= 2) {
          if (found)
            fprintf(sim_file, (bit & k3_i[byte]) ? "1" : "0");
          else if (bit & k3_i[byte]) {
            found = 1;
            fprintf(sim_file, "b1");
          }
        }
      }
      if (!found)
        fprintf(sim_file, "b0");
      fprintf(sim_file, " &\n");
    }
    sim_k3_i[0] = k3_i[0];
  }
  if (sim_data_o[2] != data_o[2] || sim_data_o[1] != data_o[1] || sim_data_o[0] != data_o[0]) {
    if (!changed) { changed = 1; fprintf(sim_file, "#%d\n", sim_count); }
    {
      int bit;
      int byte;
      int found = 0;
      for (byte = 2; byte >= 0; byte--) {
        for (bit = 128; bit >= 1; bit /= 2) {
          if (found)
            fprintf(sim_file, (bit & data_o[byte]) ? "1" : "0");
          else if (bit & data_o[byte]) {
            found = 1;
            fprintf(sim_file, "b1");
          }
        }
      }
      if (!found)
        fprintf(sim_file, "b0");
      fprintf(sim_file, " '\n");
    }
    sim_data_o[0] = data_o[0];
    sim_data_o[1] = data_o[1];
    sim_data_o[2] = data_o[2];
  }
  sim_count++;
}

